##   this file is part of the robolog libraries
##   Copyright (C) 1999, 2000 Oliver Obst
##
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
##
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

AC_DEFUN(RSC_TYPE_SOCKLEN_T,
[AC_CACHE_CHECK([for socklen_t], rsc_cv_type_socklen_t,
[
  AC_TRY_COMPILE(
  [#include <sys/socket.h>],
  [socklen_t len = 42; return len;],
  rsc_cv_type_socklen_t=yes,
  rsc_cv_type_socklen_t=no)
])
  if test $rsc_cv_type_socklen_t != yes; then
    AC_DEFINE(socklen_t, int, type of socklen_t)
  fi
])

dnl check which extension will be used for shared libs
AC_DEFUN(RSC_SO_EXT,
[
  AC_MSG_CHECKING([which extension will be used by shared libs])
  case "$ARCH" in
    *hppa*|*hpux*)
        rsc_so_ext="sl"
	if test "$GCC" = "yes"; then
	    LDFLAGS="$LDFLAGS -Wl,-E"
	else
	    LDFLAGS="$LDFLAGS -E"
	fi
	;;
    *)
	rsc_so_ext="so"
	;;
  esac
  AC_DEFINE_UNQUOTED(SO_EXT, $rsc_so_ext, shared library extension)
  AC_SUBST(SO_EXT)
  AC_MSG_RESULT($rsc_so_ext)
])

dnl check if SWI-Prolog is installed
AC_DEFUN(RSC_PROG_SWIPROLOG,
[
  AC_ARG_WITH(swiplc-dir,
    [  --with-swiplc-dir           where the SWI prolog compiler lives ],
    [
	ac_swiplc_dir="$withval"
    ])

  ac_save_path="$PATH"
  if test ! -z "$ac_swiplc_dir"; then
    PATH="$ac_gplc_dir"
  fi

  AC_PATH_PROG(SWIPLC, pl, "")
  AC_PATH_PROG(SWIPLLD, plld, "")
  PATH="$ac_save_path"
  dnl
  if test -z "$SWIPLC"; then
    AC_MSG_ERROR([SWI-Prolog not found])
  else
    if test -z "$SWIPLLD"; then
      AC_MSG_ERROR([SWI-Prolog linker not found])
    fi
  fi
])

dnl get SWI home directory
AC_DEFUN(RSC_PATH_SWIPROLOG,
[
  AC_MSG_CHECKING([where SWI-Prolog is installed])
  AC_CACHE_VAL(rsc_cv_path_swiprolog,
  [
    rsc_cv_path_swiprolog=`pl -g 'feature(home,H), write(H),halt.'`
  ])
  PATH_SWIPROLOG="$rsc_cv_path_swiprolog"
  AC_MSG_RESULT($PATH_SWIPROLOG)
  AC_SUBST(PATH_SWIPROLOG)
])

dnl Check for the flag to build a dynamic prolog library.
dnl Unfortunately, we cannot use libtool for it.
AC_DEFUN(RSC_PROLOG_LDFLAG,
[
  AC_MSG_CHECKING([what flag to use for linking the ECLiPSe library])
  AC_CACHE_VAL(rsc_cv_prolog_ldflag,
  [
    cat > conftest.$ac_ext <<EOF
    int main() {
    return 0;
    }
EOF
    ac_compile="${CXX-g++} -G conftest.$ac_ext -o conftest >/dev/null 2>conftest.out"
    AC_TRY_EVAL(ac_compile)
    ac_err=`grep 'unrecognized' conftest.out`
    if test -z "$ac_err"; then
      rsc_cv_prolog_ldflag="-G -lrobolog -lm -lg++ -lstdc++"
    else
      rsc_cv_prolog_ldflag="-shared -lrobolog -lm -lg++"
    fi
  ])
  PROLOG_LDFLAG="$rsc_cv_prolog_ldflag"
  AC_MSG_RESULT($PROLOG_LDFLAG)
  AC_SUBST(PROLOG_LDFLAG)
  rm -rf conftest*
])

dnl check if ECLiPSe library is to be build
AC_DEFUN(RSC_ECLIPSE_LIBRARY,
[
  : ${ECLIPSEDIR="/usr/local/lib/ECLiPSe"}

  AC_ARG_WITH(eclipse-dir,
    [  --with-eclipse-dir           where eclipse prolog is installed ],
    [  
       ac_eclipse_dir="$withval"
    ])

  if test -z "$ac_eclipse_dir"; then
    ac_eclipse_dir="$ECLIPSEDIR"
  fi  

  case "$target" in
    i*linux*) ECLIPSE_BIN="$ac_eclipse_dir/bin/i386_linux" \
              ECLIPSE_INCLUDE="$ac_eclipse_dir/include/i386_linux";;
    s*solaris*) ECLIPSE_BIN="$ac_eclipse_dir/bin/sparc_sunos5" \
                ECLIPSE_INCLUDE="$ac_eclipse_dir/include/sparc_sunos5";;
    *) ECLIPSE_BIN="" ECLIPSE_INCLUDE="";;
  esac
  dnl
  ac_save_path="$PATH"
  PATH="$ECLIPSE_BIN"
  AC_PATH_PROG(eclipsebin, eclipse, "")
  PATH="$ac_save_path"
  dnl
  if test ! -z "$eclipsebin"; then
    AC_MSG_CHECKING([wether to build ECLiPSe-Prolog library])
    AC_CACHE_VAL(rsc_cv_have_eclipse_h,
    [
      if test -r "$ECLIPSE_INCLUDE"/eclipse.h; then
        rsc_cv_have_eclipse_h="yes"
      else
        rsc_cv_have_eclipse_h="no"
      fi
    ])
    if test -z "$ECLIPSE_BIN"; then
      AC_MSG_RESULT([no (sparc-solaris and intel-linux only)])
    else
      if test "$rsc_cv_have_eclipse_h" = "no"; then
        AC_MSG_RESULT([no (eclipse.h not found)])
      else 
        AC_MSG_RESULT([yes])
	ECLIPSE_DIR="$ac_eclipse_dir"
	AC_SUBST(ECLIPSE_DIR)
        AC_SUBST(ECLIPSE_INCLUDE)     
      fi
    fi
  fi
])
 
dnl check if GNU robolog is to be build
AC_DEFUN(RSC_GPROLOG_LIBRARY,
[
  : ${GPROLOGDIR="/usr/local/gprolog-1.0.0"}

  AC_ARG_WITH(gprolog-dir,
    [  --with-gprolog-dir           where GNU prolog is installed ],
    [  
       ac_gprolog_dir="$withval"
    ])

  if test -z "$ac_gprolog_dir"; then
    ac_gprolog_dir="$GPROLOGDIR"
  fi

  AC_ARG_WITH(gplc-dir,
    [  --with-gplc-dir           where the GNU prolog compiler lives ],
    [
	ac_gplc_dir="$withval"
    ])

  ac_save_path="$PATH"
  if test -z "$ac_gplc_dir"; then
    PATH="$PATH":"$GPROLOGDIR"/bin
  else
    PATH="$ac_gplc_dir"
  fi

  AC_PATH_PROG(GPLC, gplc, "")
  PATH="$ac_save_path"
  dnl
  if test ! -z "$GPLC"; then
    AC_MSG_CHECKING([wether to build GNU robolog])
    AC_CACHE_VAL(rsc_cv_have_gprolog_h,
    [
      if test -r "$GPROLOGDIR"/include/gprolog.h; then
        rsc_cv_have_gprolog_h="yes"
      else
        rsc_cv_have_gprolog_h="no"
      fi
    ])
    if test -z "$GPLC"; then
      AC_MSG_RESULT([no (GNU Prolog Compiler not found)])
    else
      if test "$rsc_cv_have_gprolog_h" = "no"; then
        AC_MSG_RESULT([no (gprolog.h not found)])
      else 
        AC_MSG_RESULT([not yet])
	dnl GPROLOG_INCLUDE="$ac_gprolog_dir"/include
	dnl AC_SUBST(GPROLOG_INCLUDE)
        dnl AC_SUBST(GPLC)
      fi
    fi
  fi
])
