/* -*- Mode: C++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) May 13 2000 Oliver Obst
   communication coding functions
   $Id: coding.C,v 2.2 2000/07/13 12:23:41 fruit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#if STDC_HEADERS
# include <string.h>
#endif

#include <stdio.h>

#include "utils.h"
#include "coding.h"

// encode all characters of a string for transmission via the soccer server
// Please note: You have to checksum the string *after* encoding it.
// the recipients have to check first if they want to decode the
// message [means: if the checksum is ok], strip the checksum and THEN
// decode it again.
void
encode(char *dest, const char *src, const char *code, int n)
{
        static char d[4];

        size_t len;
        size_t nlen;
        
        size_t i, j, k;

        len = strlen(src);
        nlen = 0;

// FIXME: doesn't do anything with the 'code' yet      
        for (i=0; i < len; i++) {
                k = chr2saychr(src[i], d);
                for (j=0; j < k; j++)
                        if (nlen + j < n) dest[nlen + j] = d[j];
                nlen += k;
        }

        if (nlen < n)
                dest[nlen] = 0;
        else {
                dest[n] = 0;
//                my_error("Encode cut long message ", dest);
        }
}

// decode all characters of the input string.
void
decode(char *dest, const char *src, const char *code)
{
        size_t i, j, length;

        i = 0;
        j = 0;
// FIXME: doesn't do anything with the 'code' yet      
        length = strlen(src);
        while (i < length) {
                if (src[i] == '?') {
                        dest[j] = 16 * digit2int(src[i + 1]) +
                                digit2int(src[i + 2]);
                        i = i + 3;
                        j++;
                } else {
                        dest[j] = src[i];
                        i++;
                        j++;
                }
        }
        dest[j] = 0;
}

int
chr2saychr(unsigned char c, char s[4]) 
{

        if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') ||
            (c >= '0' && c <= '9') || (strchr(" ().+-*/<>_", c))) {
                s[0] = c;
                s[1] = 0;
                return 1;
        } else {
                s[0] = '?';
                s[1] = int2digit(c / 16);
                s[2] = int2digit(c % 16);
                s[3] = 0;
                return 3;
        }
}

int
digit2int(unsigned char c)
{
        if (c >= '0' && c <= '9') return c - '0';
        if (c >= 'A' && c <= 'F') return c - 'A' + 10;
        if (c >= 'a' && c <= 'f') return c - 'a' + 10;
        return 0;
}

char
int2digit(int n)
{
        static const char digit[] =
        { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
          'A', 'B', 'C', 'D', 'E', 'F' };

        n &= 0x0f;
        return digit[n];
}

char
digit2player(unsigned char c, char mySide, char theirSide, Unum *num) 
{
        if (c >= 'A' && c <= 'M') {
                // we can't use the 'E' since 0E0 is a number itself
                *num = c < 'E' ? c - 'A' : c - 'B';
                return mySide;
        }
        else if (c >= 'N' && c <= 'Z') {
                // we can't use the 'X' since 0X0 is interpreted as a number
                *num = c < 'X' ? c - 'N' : c - 'O';
                return theirSide;
        } else {
                *num = 0;
                return '?';
        }
}

char
player2digit(Unum num, bool myPlayer)
{
        char c;
        
        if (myPlayer) {
                c = num + 'A';
                if (c >= 'E') c++; // skip the 'E'
        } else {
                c = num + 'N';
                if (c >= 'X') c++; // skip the 'X'
        }
        return c;
}
        

// functions to add a crc checksum to a string and strip a crc checksum from a string 


// append four characters checksum to a string (encoding numbers
//  between 0 and 65535) 
char *
add_checksum(char *msg, unsigned int sum) 
{
        static char cstext[5];
#if 0
        if ((Mem->MyNumber == 11) && (strlen(msg) > 500))
                cerr << ">" << msg << "< (" << strlen(msg) << ")" << endl;
#endif
        cstext[0] = int2digit((sum & 0xF000) / 4096);
        cstext[1] = int2digit((sum & 0x0F00) / 256);
        cstext[2] = int2digit((sum & 0x00F0) / 16);
        cstext[3] = int2digit(sum & 0x000F);
        cstext[4] = '\0';

        return strcat(msg, (char *) cstext);
}

// calculate and add CRC chechsum
char *
crcsum(char *src)
{
        unsigned long int csreg;
        unsigned long int poly;  
        unsigned int bz, cz, length;

        csreg = 0;
        poly = 69665 * 256;	// 10001000000100001 * 100000000
				// poly shifted 8 bits to the left
        length = strlen(src);
        src[length+1] = 0;
        for(cz=0; cz <= length+1; cz++)
        {
                csreg = csreg + src[cz];
                for (bz=0; bz < 8; bz++)
                {
                        csreg <<= 1;
                        if (csreg >= 16777216) 
                        {
                                csreg = csreg ^ poly;
                        }
                }
        }
        csreg >>= 8;
	return add_checksum(src, csreg);
}  

// calculate and remove CRC chechsum
// if crccheck(msg) = true then transmission was OK
bool 
crccheck(char *msg) 
{
	unsigned long int csreg = 0;
        unsigned long int poly = 17834240; // 10001000000100001 * 10000000 
					   // poly shifted 8 bits to the left 
        size_t bz, cz;
	size_t length;
        unsigned int cbh = 0;
        unsigned int cbl = 0;
	unsigned char crcchar;

        length = strlen(msg);
        if (length >4)
	{
	cbh = digit2int(msg[length-4]) *16 + digit2int(msg[length-3]);
	cbl = digit2int(msg[length-2]) *16 + digit2int(msg[length-1]);
	length = length -2;
	msg[length]   = 0;
	msg[length-2] = cbh;
	msg[length-1] = cbl;
        for(cz=0; cz < length+1; cz++)
        {
                crcchar = msg[cz];
		csreg = csreg + crcchar;
                for (bz=0; bz < 8; bz++)
                {
                        csreg <<= 1;
                        if (csreg >= 16777216) /* 1000000000000000000000000 bin */
                        {
                                csreg = csreg ^ poly;
                        }
                }
        }
        msg[length-2]=0;
        csreg >>= 8;
	if (csreg == 0) return true;
        }
        return false;
}  
