/* -*- Mode: C++ -*-
   
   this file is part of the robolog libraries
   Copyright (C) May 13 2000 Oliver Obst
   agents communication stuff
   $Id: communication.C,v 2.13 2000/07/26 14:31:01 fruit Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#if STDC_HEADERS
# include <string.h>
#endif

#include "utils.h"
#include "client.h"
#include "coding.h"
#include "communication.h"

// say encoded messages
void
say_encoded(const char *msg)
{
        static char nmsg[MAXMESG];
        static char *pnmsg = nmsg;
        int i;
        encode(pnmsg, msg, "", Mem->SP_say_msg_size - 5);
        i = strlen(pnmsg);
        pnmsg[i] = Mem->MySide;
        pnmsg[i+1] = '\0';
	say(crcsum(pnmsg));
}

// hear a message (get the last heard message from memory)
void hear(char *msg)
{
	Mem->Get_Current_Msg(msg);
}


// ------------------------------------------------------------------------
// --- functions to announce objects seen
void
announce_positions()
{
        bool proceed;
        char side;
        Unum num;
        char *msg = new char[Mem->SP_say_msg_size];
        Time time = Mem->MyUpdateTime();

        sprintf(msg, "*%d %d", time.t, time.s);

        proceed = announce_ball(msg, Mem->SP_say_msg_size-4);

        side = Mem->MySide;

#if 0
        if (proceed) announce_player(msg, Mem->SP_say_msg_size-4,
                                     side, Mem->MyNumber);
#else
        while ((proceed) && (side != '?')) {
                num = 1;
                while ((proceed) && (num < 12)) {
                        proceed = announce_player(msg, Mem->SP_say_msg_size-4,
                                                  side, num);
                        num++;
                }
                side = (side == Mem->MySide) ? Mem->TheirSide : '?';
        }
#endif
        say_encoded(msg);
        delete[] msg;
}

bool
announce_ball(char *src, size_t l)
{
        size_t n = strlen(src);
//        Time time = Mem->GetBall()->GetSeenTime();
        int m;
        
        m = snprintf(src + n, l - n, "_b %.1f %.1f %.1f %.1f",
                     Mem->BallX(),
                     Mem->BallY(),
                     Mem->BallPositionValid(),
                     Mem->BallDistance());

        // check if all characters were written
        // if something went wrong snprintf returns either
        // * -1 until glibc 2.0.6   or
        // * the number of characters that would have been written
        if ((m == -1) || (m + n >= l)) {
                src[n] = '\0';
                return false;
        } 
        return true;
}

bool
announce_player(char *src, size_t l, char side, Unum num)
{
        const float minConf = 0.8;
        
//        Time time;
        size_t n = strlen(src);
        int m = 0;
        
        if ((side == Mem->MySide) && (num == Mem->MyNumber)) {
                if (Mem->MyConf() < minConf) return true;
//                time = Mem->MyPosTime();
                
                m = snprintf(src + n, l - n,
                             "_p %c %d %.1f %.1f %.1f %.1f",
                             side,
                             num,
                             Mem->MyX(),
                             Mem->MyY(),
                             Mem->MyConf(),
                             0.0);
        } else {
                if (Mem->PlayerPositionValid(side, num) < minConf)
                        return true;
//                time = Mem->GetPlayer(side,num)->GetSeenTime();

                m = snprintf(src + n, l - n,
                             "_p %c %d %.1f %.1f %.1f %.1f",
                             side,
                             num,
                             Mem->PlayerX(side, num),
                             Mem->PlayerY(side, num),
                             Mem->PlayerPositionValid(side, num),
                             Mem->PlayerDistance(side, num));
        }
        
        if ((m == -1) || (m + n >= l)) {
                src[n] = '\0';
                return false;
        }

        return true;
}



/*
	 Functions for the KQML object
*/
// Init function 
KQML_Obj::KQML_Obj()
{
	performative[0]=0;
	content[0]=0;
	strcpy(force,"tentative");
	in_reply_to[0]=0;
	language[0]=0;
	strcpy(ontology, "ROBOCUP");
	reciever[0]=0;
	reply_with[0]=0;
	sender[0]=0;
};

// save values to the Object
void KQML_Obj::elem2KQML(char *in_perf, char *in_sender, char *in_reciever, char *in_language, char *in_rw, char *in_irt, char *in_content)
{
	strcpy(performative, in_perf);
	strcpy(sender, in_sender);
	strcpy(reciever, in_reciever);
	strcpy(language, in_language);
	strcpy(reply_with, in_rw);
	strcpy(in_reply_to, in_irt);
	strcpy(content, in_content);
};

// get the KQML values from the Object
void KQML_Obj::KQML2elem(char *out_perf, char *out_sender, char *out_reciever, char *out_language, char *out_rw, char *out_irt, char *out_content)
{
	strcpy(out_perf, performative);
	strcpy(out_sender, sender);
	strcpy(out_reciever, reciever);
	strcpy(out_language, language);
	strcpy(out_rw, reply_with);
	strcpy(out_irt, in_reply_to);
	strcpy(out_content, content);
};

// build a string from the KQML object
void KQML_Obj::KQML2str(char *KQML_Phrase)
{
	strcpy (KQML_Phrase, "(");
	strcat (KQML_Phrase, performative);

	strcat (KQML_Phrase, " :sender ");
	strcat (KQML_Phrase, sender);

	if (reciever != "")
	{
		strcat (KQML_Phrase, " :reciever ");
		strcat (KQML_Phrase, reciever);
	}
	else /* no reciever defined -> broadcast to ALL */
	{
		strcat (KQML_Phrase, " :reciever ");
		strcat (KQML_Phrase, "ALL");
	}

	if (language == "") get_def_language(language);  /* no language defined -> get default Language */

	strcat (KQML_Phrase, " :language ");
	strcat (KQML_Phrase, language);

	if (reply_with !="")
	{
		strcat (KQML_Phrase, " :reply-with ");
		strcat (KQML_Phrase, reply_with);
	} /* no reply-with defined = don't use parameter */

	if (in_reply_to !="")
	{
		strcat (KQML_Phrase, " :in-reply-to ");
		strcat (KQML_Phrase, in_reply_to);
	} /* no reply-with defined = don't use parameter */

	strcat (KQML_Phrase, " :ontology ");
	strcat (KQML_Phrase, "ROBOLOG");

	strcat (KQML_Phrase, " :force ");
	strcat (KQML_Phrase, "tentative");

	strcat (KQML_Phrase, " :content ");
	strcat (KQML_Phrase, content);

	strcat (KQML_Phrase, ")");
};

// Parse a string and save the values to the object
void KQML_Obj::str2KQML (char *KQML_Phrase)
{
	char *Phrase;
	char *p; 
	size_t length;

	Phrase = KQML_Phrase; 

	p = strchr(Phrase, '(');  if(p != 0) Phrase = ++p; else return; // strip leading '('

	p = strrchr(Phrase, ')'); if(p != 0) p[0] = '\0'; else return; // strip trailing ')'

	length = strcspn(Phrase, ":\0") - 1; // get length of Performative

	strncpy (performative, Phrase, length); performative[length] = '\0'; 

	Phrase = strpbrk(Phrase, ":"); // skip to first parameter name
	
	if (Phrase != NULL) Phrase++; // skip leading ':'

	while(Phrase != NULL)
	{
		p = strpbrk(Phrase, " ") +1;
		length = strcspn(p, ":\0");  
		if (p[length] == ':') length --; // strip trailing space if necessary

		if (strncasecmp(Phrase, "sender", 6)==0) 	  {strncpy(sender, p, length); sender[length] = '\0';}
		if (strncasecmp(Phrase, "reciever", 8)==0) 	  {strncpy(reciever, p, length); reciever[length] = '\0';}
		if (strncasecmp(Phrase, "language", 8)==0) 	  {strncpy(language, p, length); language[length] = '\0';}
		if (strncasecmp(Phrase, "reply-with", 10)==0)  	  {strncpy(reply_with, p, length); reply_with[length] = '\0';}
		if (strncasecmp(Phrase, "in-reply-to", 11)==0) 	  {strncpy(in_reply_to, p, length); in_reply_to[length] = '\0';}
		if (strncasecmp(Phrase, "content", 7)==0) 	  {strncpy(content, p, length); content[length] = '\0';}

		Phrase = strpbrk(Phrase, ":\0"); 
		if (Phrase != NULL) Phrase++; // skip leading ':'

	}
}


// get default Language for KQML content
void get_def_language(char *language)
{
	// Temporary!!!
	// Get Default Language from settings file

	strcpy(language, "PROLOG");
}

// get reply code from playername and time
void get_reply_code(char *code)
{
	char name[MAX_TEAMNAME_LEN+10];

	get_my_name(name);
	sprintf(code, "%s%i", name, Mem->CurrentTime.t);
}

// get player name from teamname and number
void get_my_name(char *name)
{
	sprintf(name, "%s%d", Mem->MyTeamName, Mem->MyNumber);
}

// get teammate name from teamname and his number
void get_his_name(Unum His_Number, char *name)
{
	sprintf(name, "%s%d", Mem->MyTeamName, His_Number);
}


// KQML functions for ROBOLOG
// --------------------------

// KQML basic functions

// say a KQML Phrase
void kqml_say(char *perf, char *sender, char *reciever, char *language, char *rw, char *irt, char *content)
{
	char msg[MAXMESG];
	KQML_Obj Object;

	if (language[0] == '\0') get_def_language(language);
	if (rw[0] == '\0') get_reply_code(rw);
	if (sender[0] == '\0') get_my_name(sender);
	Object.elem2KQML(perf, sender, reciever, language, rw, irt, content); // store elements to KQML Object
	Object.KQML2str(msg); // assemble string to say from the object
	say_encoded(msg);
}

// hear a KQML Phrase
void kqml_hear(char *perf, char *sender, char *reciever, char *language, char *rw, char *irt, char *content)
{
	char msg[MAXMESG];
	KQML_Obj Object;

	hear(msg); // get current message
	Object.str2KQML(msg); // Parse message and store to KQML Object
	Object.KQML2elem(perf, sender, reciever, language, rw, irt, content); // get elements of the message
}




